"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

"""
from pyegeria import (
    INTEGRATION_GUIDS,
    TEMPLATE_GUIDS,
    default_time_out,
    disable_ssl_warnings,
    enable_ssl_check,
    is_debug,
    max_paging_size,
    NO_ELEMENTS_FOUND,
    NO_ASSETS_FOUND,
    NO_SERVERS_FOUND,
    NO_CATALOGS_FOUND,
    NO_GLOSSARIES_FOUND,
    NO_TERMS_FOUND,
    NO_CATEGORIES_FOUND,
    NO_ELEMENT_FOUND,
)